// -*- c++ -*-
/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile$

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

/*-------------------------------------------------------------------------
  Copyright 2009 Sandia Corporation.
  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
  the U.S. Government retains certain rights in this software.
-------------------------------------------------------------------------*/

// .NAME vtkInsituPVBatch
//
// .SECTION Description
//
//

#ifndef __vtkInsituPVBatch_h
#define __vtkInsituPVBatch_h

#include "vtkInsituPVBatchWin32Header.h"
#include "vtkObject.h"
#include "vtkStdString.h"

//BTX
class InsituPVBatchSingleton;
//ETX
class vtkAlgorithm;

class VTKSNL_INSITU_PVBATCH_EXPORT vtkInsituPVBatch : public vtkObject
{
public:
  static vtkInsituPVBatch *New ();
  vtkTypeRevisionMacro(vtkInsituPVBatch, vtkObject);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description: 
  // Setups up the pvbatch (works like main) 
  void Initialize (vtkStdString script);

  // Description:
  // Will shut down everything (removes pointer refs) so it all gets cleaned
  void Finalize ();

  // Description:
  // called to set the point to the vtkAlgorithm source(s)
  // created by the simulation bridge
  void SetDataSource (int index, vtkAlgorithm *source);
  vtkAlgorithm *GetDataSource (int index);

  // Description:
  // Runs the initialization script that sets up the pipeline
  void RunScript (vtkStdString script);
  
  // Description:
  // Called at the end of each step of the simulation
  void Update (int cycle, double time, double dt);
  
protected:
  vtkInsituPVBatch ();
  ~vtkInsituPVBatch ();

  InsituPVBatchSingleton *implementation;

private:
  vtkInsituPVBatch (const vtkInsituPVBatch&); // Not implemented
  void operator= (const vtkInsituPVBatch&); // Not implemented
};

#endif /* __vtkInsituPVBatch_h */
