macro( DOT_ADD_IN_SITU_TASK taskName taskLibDir taskLibraries taskLang )
  ### Always downcase the driver name
  string( TOLOWER "${taskName}" taskNameLower )

  ### Add task module if required (for interfacing with non-FORTRAN code)
  if ( taskLang STRLESS "FORTRAN" OR taskLang STRGREATER "FORTRAN" )
    set( DOT_IN_SITU_USE
      "${DOT_IN_SITU_USE}
  use ${taskNameLower}_m"
    )
    set( taskNameLowerF "${taskNameLower}_f" )
  else ( taskLang STRLESS "FORTRAN" OR taskLang STRGREATER "FORTRAN" )
    set( taskNameLowerF "${taskNameLower}" )
  endif ( taskLang STRLESS "FORTRAN" OR taskLang STRGREATER "FORTRAN" )
  ### Add task initialization code
  set( DOT_IN_SITU_INIT
    "${DOT_IN_SITU_INIT}
    call ${taskNameLowerF}_init( local_id, global_comm, &
      num_particles, c_loc(pxyz), c_loc(mass), c_loc(pmom), c_loc(pfrc) )"
  )
  ### Add per-timestep execution code
  set( DOT_IN_SITU_STEP
    "${DOT_IN_SITU_STEP}
    call ${taskNameLowerF}_step( local_id, global_comm, time )"
  )
  ### Add task completion code
  set( DOT_IN_SITU_FINI
    "${DOT_IN_SITU_FINI}
    call ${taskNameLowerF}_fini( local_id, global_comm )"
  )
  ### Declare C interface if taskLang is not FORTRAN
  if ( taskLang STRLESS "FORTRAN" OR taskLang STRGREATER "FORTRAN" )
    set( DOT_IN_SITU_IFACE
     "${DOT_IN_SITU_IFACE};${dot_BINARY_DIR}/${taskNameLower}_m.f90"
    )
    file( WRITE "${dot_BINARY_DIR}/${taskNameLower}_m.f90"
     "  module ${taskNameLower}_m
  use iso_c_binding
  implicit none

  public :: ${taskNameLowerF}_init, ${taskNameLowerF}_step, ${taskNameLowerF}_fini

  interface

  subroutine ${taskNameLowerF}_init( rank, wcomm, num_part, pcoords, pmasses, pmomenta, pforces ) &
       bind(C,NAME=\"${taskNameLower}_init\")
    ! void ${taskNameLower}_init( int rank, int wcomm, int num_particles, double* pxyz, double* mass, double* pmom, double* pfrc );
    use iso_c_binding
    integer(c_int), value :: rank
    integer(c_int), value :: wcomm
    integer(c_int), value :: num_part
    type(c_ptr), value :: pcoords
    type(c_ptr), value :: pmasses
    type(c_ptr), value :: pmomenta
    type(c_ptr), value :: pforces
  end subroutine ${taskNameLowerF}_init

  subroutine ${taskNameLowerF}_step( rank, wcomm, time ) &
       bind(C,NAME=\"${taskNameLower}_step\")
    ! void ${taskNameLower}_step( int rank, int wcomm, double time );
    use iso_c_bindinG
    integer(c_int), value :: rank
    integer(c_int), value :: wcomm
    real(c_double), value :: time
  end subroutine ${taskNameLowerF}_step

  subroutine ${taskNameLowerF}_fini( rank, wcomm ) &
       bind(C,NAME=\"${taskNameLower}_fini\")
    ! void ${taskNameLower}_fini( int rank, int wcomm );
    use iso_c_binding
    integer(c_int), value :: rank
    integer(c_int), value :: wcomm
  end subroutine ${taskNameLowerF}_fini

  end interface

  end module ${taskNameLower}_m"
    )
  endif ( taskLang STRLESS "FORTRAN" OR taskLang STRGREATER "FORTRAN" )
  ### Add libraries implementing the task to the link line
  link_directories(
    ${taskLibDir}
  )
  set( DOT_IN_SITU_LIBRARIES
    "${DOT_IN_SITU_LIBRARIES};${taskLibraries}"
  )
endmacro( DOT_ADD_IN_SITU_TASK )
