  ! ------------------------------------------------------------------
  ! GEOMETRIC UTILITIES
  ! ------------------------------------------------------------------
  module geometry_m

  contains

  ! ------------------------------------------------------------------
  ! NORMAL VECTOR TO A TRIANGLE
  !
  ! The result is not of unit length but the return value of the
  ! function is the length of the normal vector.
  real function triangle_normal( norm, x0, x1, x2 )
  implicit none
  real, intent(out) :: norm(3)
  real, intent(in) :: x0(3)
  real, intent(in) :: x1(3)
  real, intent(in) :: x2(3)

  real edg0(3)
  real edg1(3)
  real nmag
  integer i
  do i = 1,3
    edg0(i) = x1(i) - x0(i)
    edg1(i) = x2(i) - x0(i)
  enddo
  norm(1) = edg0(2)*edg1(3) - edg0(3)*edg1(2)
  norm(2) = edg0(3)*edg1(1) - edg0(1)*edg1(3)
  norm(3) = edg0(1)*edg1(2) - edg0(2)*edg1(1)
  nmag = 0.
  do i = 1,3
    nmag = nmag + norm(i) * norm(i)
  enddo
  nmag = sqrt( nmag )
  triangle_normal = nmag
  end function triangle_normal
  ! ------------------------------------------------------------------

  ! ------------------------------------------------------------------
  ! NORMAL VECTOR TO A TRIANGLE
  !
  ! The result is not of unit length but the return value of the
  ! function is the length of the normal vector.
  real function triangle_unit_normal( norm, x0, x1, x2 )
  implicit none
  real, intent(out) :: norm(3)
  real, intent(in) :: x0(3)
  real, intent(in) :: x1(3)
  real, intent(in) :: x2(3)

  integer i

  triangle_unit_normal = triangle_normal( norm, x0, x1, x2 )
  if ( triangle_unit_normal .gt. 1e-20 ) then
    do i = 1,3
      norm(i) = norm(i) / triangle_unit_normal
    enddo
  endif
  end function triangle_unit_normal
  ! ------------------------------------------------------------------

  ! ------------------------------------------------------------------
  real function triangle_area( x0, x1, x2 )
  implicit none
  real, intent(in) :: x0(3)
  real, intent(in) :: x1(3)
  real, intent(in) :: x2(3)

  real norm(3)
  triangle_area = triangle_normal( norm, x0, x1, x2 )
  triangle_area = triangle_area / 2. 
  end function triangle_area

  end module geometry_m
